@@ -14,6 +14,7 @@ from liquid.token import TOKEN_EOF
 from liquid.token import TOKEN_TAG
 from liquid.token import TOKEN_EXPRESSION
 
+from liquid.ast import ChildNode
 from liquid.ast import Node
 from liquid.ast import BlockNode
 from liquid.ast import ConditionalBlockNode
@@ -141,6 +142,35 @@ class IfNode(Node):
 
         return rendered
 
+    def children(self) -> List[ChildNode]:
+        _children = [
+            ChildNode(
+                linenum=self.tok.linenum,
+                node=self.consequence,
+                expression=self.condition,
+            )
+        ]
+        _children.extend(
+            [
+                ChildNode(
+                    linenum=alt.tok.linenum,
+                    node=alt.block,
+                    expression=alt.condition,
+                )
+                for alt in self.conditional_alternatives
+            ]
+        )
+        if self.alternative:
+            _children.append(
+                ChildNode(
+                    linenum=self.alternative.tok.linenum,
+                    node=self.alternative,
+                    expression=None,
+                )
+            )
+
+        return _children
+
 
 class IfTag(Tag):
     """The built-in "if" tag."""
