@@ -45,6 +45,10 @@ class Expression(ABC):
         """An async version of :meth:`liquid.expression.Expression.evaluate`."""
         return self.evaluate(context)
 
+    def children(self) -> List[Expression]:
+        """Return a list of child expressions."""
+        raise NotImplementedError(f"{self.__class__.__name__}.children")
+
 
 class Nil(Expression):
     __slots__ = ()
@@ -61,6 +65,9 @@ class Nil(Expression):
     def evaluate(self, context: Context) -> None:
         return None
 
+    def children(self) -> List[Expression]:
+        return []
+
 
 NIL = Nil()
 
@@ -84,6 +91,9 @@ class Empty(Expression):
     def evaluate(self, context: Context) -> Empty:
         return self
 
+    def children(self) -> List[Expression]:
+        return []
+
 
 EMPTY = Empty()
 
@@ -98,7 +108,6 @@ class Blank(Expression):
             return True
         if isinstance(other, Blank):
             return True
-
         return False
 
     def __repr__(self) -> str:  # pragma: no cover
@@ -110,6 +119,9 @@ class Blank(Expression):
     def evaluate(self, context: Context) -> Blank:
         return self
 
+    def children(self) -> List[Expression]:
+        return []
+
 
 BLANK = Blank()
 
@@ -131,6 +143,9 @@ class Continue(Expression):
     def evaluate(self, context: Context) -> int:
         return 0
 
+    def children(self) -> List[Expression]:
+        return []
+
 
 CONTINUE = Continue()
 
@@ -156,6 +171,9 @@ class Literal(Expression, Generic[T]):
     def evaluate(self, context: Context) -> object:
         return self.value
 
+    def children(self) -> List[Expression]:
+        return []
+
 
 class Boolean(Literal[bool]):
     __slots__ = ()
@@ -239,7 +257,7 @@ class RangeLiteral(Expression):
         )
 
     def __str__(self) -> str:
-        return f"{self.start}..{self.stop}"
+        return f"({self.start}..{self.stop})"
 
     def __repr__(self) -> str:  # pragma: no cover
         return f"RangeLiteral(start={self.start}, stop={self.stop})"
@@ -266,14 +284,18 @@ class RangeLiteral(Expression):
         return range(start, stop + 1)
 
     def evaluate(self, context: Context) -> range:
-        start: Any = self.start.evaluate(context)
-        stop: Any = self.stop.evaluate(context)
-        return self._make_range(start, stop)
+        return self._make_range(
+            self.start.evaluate(context), self.stop.evaluate(context)
+        )
 
     async def evaluate_async(self, context: Context) -> range:
-        start: Any = await self.start.evaluate_async(context)
-        stop: Any = await self.stop.evaluate_async(context)
-        return self._make_range(start, stop)
+        return self._make_range(
+            await self.start.evaluate_async(context),
+            await self.stop.evaluate_async(context),
+        )
+
+    def children(self) -> List[Expression]:
+        return [self.start, self.stop]
 
 
 class IdentifierPathElement(Literal[Union[int, str]]):
@@ -309,7 +331,6 @@ class Identifier(Expression):
 
     def __str__(self) -> str:
         buf = []
-
         for elem in self.path:
             if isinstance(elem, Identifier):
                 buf.append(f"[{elem}]")
@@ -328,6 +349,9 @@ class Identifier(Expression):
         path: List[Any] = [await elem.evaluate_async(context) for elem in self.path]
         return await context.get_async(path)
 
+    def children(self) -> List[Expression]:
+        return [el for el in self.path if isinstance(el, Expression)]
+
 
 class PrefixExpression(Expression):
     __slots__ = ("operator", "right")
@@ -358,12 +382,13 @@ class PrefixExpression(Expression):
         raise LiquidTypeError(f"unknown operator {self.operator}")
 
     def evaluate(self, context: Context) -> object:
-        right = self.right.evaluate(context)
-        return self._evaluate(right)
+        return self._evaluate(self.right.evaluate(context))
 
     async def evaluate_async(self, context: Context) -> object:
-        right = await self.right.evaluate_async(context)
-        return self._evaluate(right)
+        return self._evaluate(await self.right.evaluate_async(context))
+
+    def children(self) -> List[Expression]:
+        return [self.right]
 
 
 class InfixExpression(Expression):
@@ -397,14 +422,19 @@ class InfixExpression(Expression):
         return f"({self.left} {self.operator} {self.right})"
 
     def evaluate(self, context: Context) -> object:
-        left = self.left.evaluate(context)
-        right = self.right.evaluate(context)
-        return compare(left, self.operator, right)
+        return compare(
+            self.left.evaluate(context), self.operator, self.right.evaluate(context)
+        )
 
     async def evaluate_async(self, context: Context) -> object:
-        left = await self.left.evaluate_async(context)
-        right = await self.right.evaluate_async(context)
-        return compare(left, self.operator, right)
+        return compare(
+            await self.left.evaluate_async(context),
+            self.operator,
+            await self.right.evaluate_async(context),
+        )
+
+    def children(self) -> List[Expression]:
+        return [self.left, self.right]
 
 
 class Filter:
@@ -497,6 +527,9 @@ class BooleanExpression(Expression):
     async def evaluate_async(self, context: Context) -> bool:
         return is_truthy(await self.expression.evaluate_async(context))
 
+    def children(self) -> List[Expression]:
+        return [self.expression]
+
 
 class FilteredExpression(Expression):
     __slots__ = ("expression", "filters")
@@ -600,6 +633,13 @@ class FilteredExpression(Expression):
 
         return result
 
+    def children(self) -> List[Expression]:
+        _children = [self.expression]
+        for fltr in self.filters:
+            _children.extend(fltr.args)
+            _children.extend(fltr.kwargs.values())
+        return _children
+
 
 class AssignmentExpression(Expression):
     __slots__ = ("name", "expression")
@@ -634,6 +674,9 @@ class AssignmentExpression(Expression):
         )
         return ""
 
+    def children(self) -> List[Expression]:
+        return [self.expression]
+
 
 LoopArgument = Union[IntegerLiteral, FloatLiteral, Identifier, Continue, Nil]
 
@@ -816,6 +859,14 @@ class LoopExpression(Expression):
             context=context,
         )
 
+    def children(self) -> List[Expression]:
+        return [
+            self.iterable,
+            self.limit,
+            self.offset,
+            self.cols,
+        ]
+
 
 Number = Union[int, float]
 
